﻿/*-------------------------------------------------------------------------
 Trace and control cryptographic module(AES) of smartcard on SASEBO-W(VCP).

 File name   : Tracer_SASEBO_W_VCP_Card.cs
 Version     : Version 1.0
 Created     : Apr/01/2011
 Last update : Jun/27/2011
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2011 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://staff.aist.go.jp/akashi.satoh/SASEBO/en/).
 -------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;

namespace Tracer
{
	//================================================================ Tracer_SASEBO_W_VCP_Card
  public class Tracer_SASEBO_W_VCP_Card : Tracer
  {
		//************************************************ Variable
    CipherModule.SASEBO_W_VCP_Card module;
    int num_trace;
    byte[] key;

		//************************************************ Method
		//------------------------------------------------ Constructor
		public Tracer_SASEBO_W_VCP_Card (CipherModule.SASEBO_W_VCP_Card module, int num_trace, string key_st)
    {
      this.module    = module;
			this.num_trace = num_trace;

      key = new byte[16];
      for (int i=0; i<16; i++) key[i] = Convert.ToByte(key_st.Substring(i * 3, 2), 16);

      worker = new BackgroundWorker();
      worker.WorkerReportsProgress      = true;
      worker.WorkerSupportsCancellation = true;
      worker.DoWork += new DoWorkEventHandler(worker_DoWork);
    }


		//************************************************ BackgroudWorker
    //------------------------------------------------ worker_DoWork()
    private void worker_DoWork(object sender, DoWorkEventArgs e)
    {
      e.Cancel = false;

      CipherModule.IBlockCipher cipher_hw = (CipherModule.IBlockCipher)module;
      CipherTool.IBlockCipher   cipher_sw = (CipherTool.IBlockCipher) new CipherTool.AES();

      int progress = 0;
      e.Cancel = trace_loop_AES(cipher_hw, cipher_sw, ref progress);
    }

    //------------------------------------------------ worker_DoWork()
		private bool trace_loop_AES(CipherModule.IBlockCipher hw, CipherTool.IBlockCipher sw, ref int progress)
		{
      Random rand = new Random();

      int loop     = 0;
      bool error   = false;

      byte[] text_in  = new byte[16];
      byte[] text_out = new byte[16];
      byte[] text_ans = new byte[16];

      // Initialization
      hw.setKey(key, 16); // Hardware
      sw.setKey(key);  // Software 

      // Main loop
      progress = 0;
      worker.ReportProgress(progress, (object)(new TracerReport(loop, text_in, text_out, text_ans)));
      for (loop = 0; loop < num_trace; loop++)
      {
        progress = (int)(100.0d * loop / num_trace);

        for (int i=0; i<16; i++) text_in[i] = (byte)rand.Next(256);
        // Generate answer
        text_ans = sw.encrypt(text_in);

        // Cryptographic processing
        hw.writeText(text_in,  16);
        hw.readText (text_out, 16);

        // Check result
        for (int i=0; i < 16; i++) if (text_out[i] != text_ans[i]) error = true;
        worker.ReportProgress(progress, (object)(new TracerReport(loop, text_in, text_out, text_ans)));

        // Caceled ?
        if (worker.CancellationPending || error) break;
      } //endof loop

      if (worker.CancellationPending) error = true;
      
      worker.ReportProgress(progress, (object)(new TracerReport(loop, text_in, text_out, text_ans)));
      return error;
    }

  }
}
